﻿using Microsoft.EntityFrameworkCore;
using Netina.Common.Models.Api;
using Netina.Common.Models.Exception;
using Netina.Domain.CommandQueries.Commands;
using Netina.Domain.Entities.Products;
using Netina.Repository.Repositories.Base.Contracts;

namespace Netina.Repository.Handlers.Reviews;

public class DeleteReviewCommandHandler : IRequestHandler<DeleteReviewCommand,bool>
{
    private readonly IRepositoryWrapper _repositoryWrapper;

    public DeleteReviewCommandHandler(IRepositoryWrapper repositoryWrapper)
    {
        _repositoryWrapper = repositoryWrapper;
    }
    public async Task<bool> Handle(DeleteReviewCommand request, CancellationToken cancellationToken)
    {
        var review = await _repositoryWrapper.SetRepository<Review>().TableNoTracking
            .FirstOrDefaultAsync(r => r.Id == request.Id, cancellationToken);
        if (review == null)
            throw new AppException("Review not found", ApiResultStatusCode.NotFound);

        _repositoryWrapper.SetRepository<Review>().Delete(review);
        await _repositoryWrapper.SaveChangesAsync(cancellationToken);
        return true;
    }
}